require 'clean_errors'

RSpec.describe CleanErrors::ValidationError do
  subject { CleanErrors::ValidationError('ResourceNotFound') }

  it 'should render json' do
    expected_value = {
      type: 'ResourceNotFound',
      message: 'translation missing: en.errors.ResourceNotFound'
    }
    expect(subject.to_json).to eq(expected_value.to_json)
  end

  context 'for Error' do
    subject { CleanErrors::Error('GenericError') }

    it 'should render json' do
      expected_value = {
        type: 'GenericError',
        message: 'translation missing: en.errors.GenericError'
      }
      expect(subject.to_json).to eq(expected_value.to_json)
    end
  end

  context 'for different path' do
    before { CleanErrors.translation_path = 'mypath' }
    after { CleanErrors.translation_path = nil }

    it 'should render json' do
      expected_value = {
        type: 'ResourceNotFound',
        message: 'translation missing: en.mypath.ResourceNotFound'
      }
      expect(subject.to_json).to eq(expected_value.to_json)
    end
  end
end
